@@ -3,7 +3,7 @@ package MVC::Neaf::Request;
 use strict;
 use warnings;
 
-our $VERSION = 0.1901;
+our $VERSION = 0.20;
 
 =head1 NAME
 
@@ -46,6 +46,9 @@ use Carp;
 use URI::Escape;
 use Encode;
 use HTTP::Headers;
+use Time::HiRes ();
+use Sys::Hostname ();
+use Digest::MD5 qw(md5_base64);
 
 use MVC::Neaf::Util qw(http_date run_all_nodie canonize_path);
 use MVC::Neaf::Upload;
@@ -439,8 +442,18 @@ Use url_param() (see below) if you intend to mix GET/POST parameters.
 B<NOTE> param() ALWAYS returns a single value, even in list context.
 Use multi_param() (see below) if you really want a list.
 
-B<NOTE> Behaviour changed since 0.11 - missing default value no more
-interpreted as '', returns undef.
+B<NOTE> param() has I<nothing to do> with getting parameter list from request.
+Instead, use form with wildcards:
+
+    neaf form => "my_form" => [ [ 'guest\d+' => '.*'], [ 'arrival\d+' => '.*' ] ],
+        engine => 'Wildcard';
+
+    # later in controller
+    my $guests = $req->form("my_form");
+    $guests->fields; # has guest1 & arrival1, guest2 & arrival2 etc
+    $guests->error;  # hash with values that didn't match regexp
+
+See L<MVC::Neaf::X::Form::Wildcard>.
 
 =cut
 
@@ -555,18 +568,39 @@ sub set_param {
 
 Apply validator to raw params and return whatever it returns.
 
-Validator MUST either be a CODEREF,
-or be an object with validate() method accepting a hashref.
+A validator MUST be an object with C<validate> method, or a coderef,
+or a symbolic name registered earlier via C<neaf form ...>.
 
-See L<MVC::Neaf::X::Form> for details on Neaf's built in validator.
+Neaf's own validators, C<MVC::Neaf::X::Form> and C<MVC::Neaf::X::Form::LIVR>,
+will return a C<MVC::Neaf::X::Form::Data> object with the following methods:
+
+=over
+
+=item * is_valid - tells whether data passed validation
+
+=item * error - hash of errors, can also be modified if needed:
+
+    $result->error( myfield => "Correct, but not found in database" );
+
+=item * data - hash of valid, cleaned data
+
+=item * raw - hash of data entered initially, may be useful to display
+input form again.
+
+=back
+
+You are encouraged to use this return format from your own validation class
+or propose improvements.
 
 =cut
 
 sub form {
     my ($self, $validator) = @_;
 
-    $self->_croak("Validator must be a CODEREF or an object")
-        unless ref $validator;
+    if (!ref $validator) {
+        $validator = $self->{_neaf} && $self->{_neaf}->get_form( $validator )
+            || $self->_croak("Unknown form name $validator");
+    };
 
     if (ref $validator eq 'CODE') {
         return $validator->( $self->_all_params );
@@ -641,51 +675,14 @@ sub body {
     return $self->{body};
 };
 
-=head2 set_default( key => $value, ... )
-
-Set default values for your return hash.
-May be useful inside C<pre_route> and C<pre_logic> hooks.
-
-Returns self.
-
-B<DEPRECATED>. Use path-based defaults and/or $request->stash instead.
-This will be removed in v.0.20+.
-
-=cut
-
-sub set_default {
-    my ($self, %args) = @_;
-
-    # TODO 0.20 remove
-    carp "NEAF:set_default() DEPRECATED. Use path-based defaults or stash()";
-    foreach (keys %args) {
-        defined $args{$_}
-            ? $self->{defaults}{$_} = $args{$_}
-            : delete $self->{defaults}{$_};
-    };
-
-    return $self;
-};
-
-=head2 get_default()
-
-Returns a hash of previously set default values.
-
-B<EXPERIMANTAL>. API and naming subject to change.
-
-=cut
-
-sub get_default {
-    my $self = shift;
-
-    return $self->{defaults} || {};
-};
-
-=head2 upload( "name" )
+=head2 upload_utf8( "name" )
 
 Returns an L<MVC::Neaf::Upload> object corresponding to an uploaded file,
 if such uploaded file is present.
 
+All data read from such upload will be converted to unicode,
+raising an exception if decoding ever fails.
+
 An upload object has at least C<handle> and C<content> methods to work with
 data:
 
@@ -706,20 +703,46 @@ or just
         };
     };
 
+=head2 upload_raw( "name" )
+
+Like above, but no decoding whatsoever is performed.
+
+=head2 upload( "name" )
+
+B<DEPRECATED>. Same as upload_raw.
+
 =cut
 
-sub upload {
-    my ($self, $id) = @_;
+sub upload_utf8 {
+    my ($self, $name) = @_;
+    return $self->_upload( id => $name, utf8 => 1 );
+};
 
+sub upload_raw {
+    my ($self, $name) = @_;
+    return $self->_upload( id => $name, utf8 => 0 );
+};
+
+sub upload {
+    my ($self, $name) = @_;
+
+    carp "NEAF: req->upload() is DEPRECATED, use upload_utf8 or upload_raw instead";
+    return $self->_upload( id => $name, utf8 => 0 );
+};
+
+# TODO 0.30 do something about upload's content type
+# TODO 0.30 and btw, restrict content types!
+sub _upload {
+    my ($self, %args) = @_;
+
+    my $id = $args{id}
+        or $self->_croak( "upload name is required for upload" );
     # caching undef as well, so exists()
     if (!exists $self->{uploads}{$id}) {
         my $raw = $self->do_get_upload( $id );
-        # This would create NO upload objects for objects
-        # And also will return undef as undef - just as we want
-        #    even though that's side effect
         $self->{uploads}{$id} = (ref $raw eq 'HASH')
-            ? MVC::Neaf::Upload->new( %$raw, id => $id )
-            : $raw;
+            ? MVC::Neaf::Upload->new( %$raw, %args )
+            : undef;
     };
 
     return $self->{uploads}{$id};
@@ -1379,15 +1402,29 @@ sub clear {
 Lazily fetch unique request id. These are guaranteed to be unique
 on a given machine within a reasonable timeframe.
 
+Current id-generation mechanism involves url-safe md5_base64 C<[-_]>,
+but this MAY change in the future.
+
+B<CAUTION> Don't use this id for anything secure.
+Use L<MVC::Neaf::X::Session>'s ids instead.
+This one is just for information.
+
 =cut
 
-my $lastid;
+# TODO 0.30 provide a configurable mechanism to generate ids
+my $host = Sys::Hostname::hostname();
+my $lastid = 0;
 sub id {
     my $self = shift;
 
-    # Technically it is possible to repeat it by running a new process
-    # in the same second as the same pid... But who would?
-    return $self->{id} ||= unpack "H*", pack "N*", $$, CORE::time, ++$lastid;
+    # We don't really need to protect anything here
+    # Just avoid accidental matches for which md5 seems good enough
+    return $self->{id} ||= do {
+        my $str = md5_base64( join ".", $host, $$, Time::HiRes::time, ++$lastid );
+        $lastid = 0 unless $lastid < 4_000_000_000;
+        $str =~ tr/+\//-_/;
+        $str;
+    };
 };
 
 =head2 set_id( $new_value )
@@ -1412,11 +1449,46 @@ sub set_id {
     return $self;
 };
 
+=head2 log_error( $message )
+
+Log an error message, annotated with request id and the route being processed.
+
+Currently works via warn, but this may change in the future.
+
+B<EXPERIMENTAL.> This feature is still under development.
+
+One can count on C<log_error> to be available in the future and do
+some king of logging.
+
+=cut
+
+sub log_error {
+    my ($self, $msg) = @_;
+    $self->do_log_error( $self->_log_mark($msg) );
+};
+
+sub _where {
+    my $self = shift;
+
+    return $self->{script_name} || "pre_route";
+};
+
+sub _log_mark {
+    my ($self, $msg) = @_;
+
+    $msg ||= Carp::shortmess( "Something bad happened" );
+    $msg =~ s/\s*$//s;
+
+    return "req_id=".$self->id." in ".$self->_where.": $msg";
+};
+
+# See do_log_error below
+
 =head2 endpoint_origin
 
-Returns file:line where the route was created.
+Returns file:line where controller was defined.
 
-B<EXPERIMENTAL>. Name and semantics subject to change.
+B<DEPRECATED>. Do not use.
 
 =cut
 
@@ -1507,9 +1579,13 @@ They shall not generally be called directly inside the app.
 
 =item * do_reply( $status, $content ) - write reply to client
 
-=item * do_write
+=item * do_reply( $status ) - only send headers to client
 
-=item * do_close
+=item * do_write( $data )
+
+=item * do_close()
+
+=item * do_log_error()
 
 =back
 
@@ -1533,6 +1609,13 @@ foreach (qw(
 # by default, just skip - the handle will auto-close anyway at some point
 sub do_close { return 1 };
 
+# by default, write to STDERR - ugly but at least it will get noticed
+sub do_log_error {
+    my ($self, $msg) = @_;
+
+    warn "ERROR $msg\n";
+};
+
 sub _croak {
     my ($self, $msg) = @_;
 
@@ -1541,4 +1624,55 @@ sub _croak {
     croak( (ref $self || $self)."->$where: $msg" );
 };
 
+=head1 DEPRECATED METHODS
+
+Some methods become obsolete during Neaf development.
+Anything that is considered deprecated will continue to be supported
+I<for at least three minor versions> after official deprecation
+and a corresponding warning being added.
+
+Please keep an eye on C<Changes> though.
+
+=head2 set_default( key => $value, ... )
+
+As of v.0.20 this dies.
+
+USe path-based defaults, or stash().
+
+=cut
+
+sub set_default {
+    my ($self, %args) = @_;
+
+    # TODO 0.30 remove completely
+    $self->_croak( "DEPRECATED. Use path-based defaults or stash()" );
+};
+
+=head2 get_default()
+
+As of v.0.20 this dies.
+
+USe path-based defaults, or stash().
+
+=cut
+
+sub get_default {
+    my $self = shift;
+
+    # TODO 0.30 remove completely
+    $self->_croak( "DEPRECATED. Use path-based defaults or stash()" );
+};
+
+=head1 LICENSE AND COPYRIGHT
+
+Copyright 2016-2017 Konstantin S. Uvarin L<khedin@cpan.org>.
+
+This program is free software; you can redistribute it and/or modify it
+under the terms of either: the GNU General Public License as published
+by the Free Software Foundation; or the Artistic License.
+
+See http://dev.perl.org/licenses/ for more information.
+
+=cut
+
 1;
