@@ -3,7 +3,7 @@ package MVC::Neaf::View::JS;
 use strict;
 use warnings;
 
-our $VERSION = 0.1901;
+our $VERSION = 0.20;
 
 =head1 NAME
 
@@ -24,7 +24,7 @@ instead or rendering a template.
 
 =cut
 
-use JSON;
+use JSON::MaybeXS;
 
 use parent qw(MVC::Neaf::View);
 
